using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using Wilco.SyntaxHighlighting;

namespace SyntaxHighlight
{
    [Serializable]
    internal sealed class Configuration
    {
        public const string HtmlParsedBlock = "{highlighted_source_code}";

        private string _language = "c#";
        private string _template = "<pre>" + HtmlParsedBlock + "</pre>";
        private string _configPath;

        private static Configuration _instance;
        private static List<KeyValuePair<string, string>> _availableLanguages;

        static Configuration()
        {   
            _instance = new Configuration();
            _instance = _instance.Load();

            // List available languages
            _availableLanguages = new List<KeyValuePair<string, string>>();

            HighlighterCollection langs = Register.Instance.Highlighters;
            for (int i = 0; i < langs.Count; i++)
            {
                _availableLanguages.Add(new KeyValuePair<string, string>(langs[i].Name.ToLower(), langs[i].FullName));
            }
        }

        private Configuration()
        {
            _configPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "BuayaCorp");
            if (!Directory.Exists(_configPath))
            {
                Directory.CreateDirectory(_configPath);
            }
            _configPath = Path.Combine(_configPath, "config.bc"); 
        }

        public static Configuration GetInstance()
        {
            return _instance;
        }

        public static List<KeyValuePair<string, string>> GetLanguages()
        {
            return _availableLanguages;            
        }

        public string Language
        {
            get { return _language; }
            set { _language = value; }
        }
        public string Template
        {
            get { return _template; }
            set { _template = value; }
        }

        public void Save()
        {
            if (null == _configPath)
                throw new ArgumentNullException("path", "The path to the configuration file can not be null.");

            FileStream stream = null;
            try
            {
                stream = new FileStream(_configPath, FileMode.Create);
                IFormatter formatter = new BinaryFormatter();
                formatter.Serialize(stream, this);
            }
            finally
            {
                //Close stream in finally block to prevent memory leaks on errors.
                //
                if (null != stream)
                    stream.Close();
            }
        }

        public Configuration Load()
        {
            FileStream stream = null;
            Configuration config;
            try
            {
                stream = new FileStream(_configPath, FileMode.Open); ;
                IFormatter formatter = new BinaryFormatter();
                config = formatter.Deserialize(stream) as Configuration;

                if (config == null)
                    config = _instance;
            }
            catch
            {
                return _instance;
            }
            finally
            {
                if (null != stream)
                    stream.Close();
            }

            return config; 
        }
    }
}
