using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using Wilco.SyntaxHighlighting;
using Wilco.Windows.SyntaxHighlighting;

namespace SyntaxHighlight
{
    public partial class InputForm : Form
    {
        private HighlighterBase highlighter;
        private RtfParser parser;

        private Configuration config = Configuration.GetInstance();

        public InputForm()
        {
            InitializeComponent();

            highlighter = Register.Instance.Highlighters[config.Language];
            parser = new Wilco.Windows.SyntaxHighlighting.RtfParser(this.outputBox);

            highlighter.Parser = parser;

            cbLanguages.DataSource = Configuration.GetLanguages();
            cbLanguages.SelectedValue = config.Language;

            lblVersion.Text += Assembly.GetExecutingAssembly().GetName().Version.ToString();
        }

        private void Highlight()
        {
            if (this.outputBox.EnablePaint)
            {
                this.outputBox.EnablePaint = false;
                int start = this.outputBox.SelectionStart, length = this.outputBox.SelectionLength;
                this.outputBox.SuspendLayout();
                this.outputBox.Rtf = this.highlighter.Parse(this.outputBox.Text);
                this.outputBox.SelectionStart = start;
                this.outputBox.SelectionLength = length;
                this.outputBox.EnablePaint = true;
            }
        }

        private void outputBox_TextChanged(object sender, EventArgs e)
        {
            Highlight();
        }

        private void cbLanguages_SelectedIndexChanged(object sender, EventArgs e)
        {
            string lang = config.Language;
            if (cbLanguages.SelectedValue != null)
                lang = cbLanguages.SelectedValue.ToString();

            highlighter = Register.Instance.Highlighters[lang];
            highlighter.Parser = parser;

            Highlight();
        }

        public string GetHtml()
        {
            return GetHtml(outputBox.Text);
        }

        public string GetHtml(string sourceCode)
        {
            highlighter.Parser = new HtmlParser();

            string result = config.Template.Replace(
                Configuration.HtmlParsedBlock,
                highlighter.Parse(sourceCode)
                );
            return result;
        }

        private void buttonConfig_Click(object sender, EventArgs e)
        {
            using (Options form = new Options())
            {
                form.StartPosition = FormStartPosition.CenterParent;
                form.ShowDialog(this);
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(outputBox.SelectedText))
                Clipboard.SetDataObject(outputBox.SelectedText, true);
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            outputBox.SelectAll();
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            IDataObject iData = Clipboard.GetDataObject();

            if (iData.GetDataPresent(DataFormats.Text))
                outputBox.SelectedText = (string)iData.GetData(DataFormats.Text);
        }
    }
}